@@ -1,7 +1,5 @@
 from referencing._core import (  # noqa: F401
+    OPAQUE_SPECIFICATION,
     IdentifiedResource,
-    OpaqueSpecification as _Opaque,
     Registry,
 )
-
-OPAQUE_SPECIFICATION = _Opaque()
